--[[
    Helper functions
]]--
--[[
Copyright (c) 2008-2015, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibLordFarlander-ItemList-2.0"
local MINOR_VERSION = 20150701033005

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true )
if( not LibLordFarlander ) then
    error( MAJOR_VERSION .. " requires LibLordFarlander-2.0" )
    return
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
if( not TR ) then
    error( MAJOR_VERSION .. " requires LibLordFarlander-TableRecycler-1.0" )
    return
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true )

local LibLordFarlander_ItemList, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION )
if( not LibLordFarlander_ItemList ) then
    return
end--if

local function GetName_Spell( item )
    local name = GetSpellInfo( item.spellID )

    return name
end--GetName_Spell( item )

local function GetName_InventoryItem( item )
    local name = GetItemInfo( GetInventoryItemLink( "player", item.invslot ) )

    return name
end--GetName_InventoryItem( item )

local function GetName_Item( item )
    local name = GetItemInfo( GetItemInfo( item.itemID ) )

    return name
end--GetName_Item( item )

local function GetName_BagItem( item )
    local name = GetItemInfo( GetContainerItemLink( item.bag, item.slot ) )

    return name
end--GetName_BagItem( item )

local function GetName_BattlePet( item )
    local _, customName, _, _, _, _, _, petName = C_PetJournal.GetPetInfoByPetID( item.fullPetID or ("0x%016x"):format( item.petID ) )

    return customName or petName or item.fullPetID or ("0x%016x"):format( item.petID )
end--GetName_BattlePet( item )

local function GetName_Mount( item )
    local creatureName = C_MountJournal.GetMountInfo( item.mountID )

    return creatureName
end--GetName_Mount( item )

local function GetName_Toy( item )
    local _, toyName = CToyBox.GetToyInfo( item.toyID )

    return toyName
end--GetName_Toy( item )

local function GetName_Follower( item )
    local info = C_Garrison.GetFollowerInfo( item.followerID )

    if( info ) then
        return info.name
    end--if
    return nil
end--GetName_Follower( item )

local function GetLink_Spell( item )
    return GetSpellLink( item.spellID )
end--GetLink_Spell( item )

local function GetLink_InventoryItem( item )
    return GetInventoryItemLink( "player", item.invslot )
end--GetLink_InventoryItem( item )

local function GetLink_Item( item )
    return LibLordFarlander.GetItemLink( item.itemID )
end--GetLink_Item( item )

local function GetLink_BagItem( item )
    return GetContainerItemLink( item.bag, item.slot )
end--GetLink_BagItem( item )

local function GetLink_BattlePet( item )
    return C_PetJournal.GetBattlePetLink( ("0x%016x"):format( item.petID ) )
end--GetLink_BattlePet( item )

local function GetLink_Mount( item )
    local _, creatureSpellID = C_MountJournal.GetMountInfo( item.mountID )

    return GetSpellLink( creatureSpellID )
end--GetLink_Mount( item )

local function GetLink_Toy( item )
    return C_ToyBox.GetToyLink( self.toyID )
end--GetLink_Toy( item )

local function GetLink_Follower( item )
    return C_Garrison.GetFollowerLink( self.followerID )
end--GetLink_Follower( item )

local function GetTexture_Spell( item )
    local _, _, texture = GetSpellInfo( item.spellID )

    return texture
end--GetTexture_Spell( item )

local function GetTexture_IventoryItem( item )
    return GetInventoryItemTexture( "player", item.invslot )
end--GetTexture_IventoryItem( item )

local function GetTexture_Item( item )
    local _, _, _, _, _, _, _, _, _, texture = GetItemInfo( item.itemID )

    return texture
end--GetTexture_Item( item )

local function GetTexture_BagItem( item )
    local _, _, _, _, _, _, _, _, _, texture = GetItemInfo( GetContainerItemLink( item.bag, item.slot ) )

    return texture
end--GetTexture_BagItem( item )

local function GetTexture_BattlePet( item )
    local _, _, _, _, _, _, _, _, texture = C_PetJournal.GetPetInfoByPetID( item.fullPetID or ("0x%016x"):format( item.petID ) )

    return texture
end--GetTexture_BattlePet( item )

local function GetTexture_Mount( item )
    local _, _, texture = C_MountJournal.GetMountInfo( item.mountID )

    return texture
end--GetTexture_Mount( item )

local function GetTexture_Toy( item )
    local _, _, texture = C_ToyBox.GetToyInfo( item.toyID )

    return texture
end--GetTexture_Toy( item )

local function GetTexture_Follower( item )
    local info = C_Garrison.GetFollowerInfo( item.followerID )

    if( info ) then
        return info.portraitIconID
    end--if
    return nil
end--GetTexture_Follower( item )

function LibLordFarlander_ItemList.BuildBasicItem()
    return TR:GetTable( "GetName", LibLordFarlander_ItemList.GetNameFromListEntry, "GetType", LibLordFarlander_ItemList.GetListEntryType, "GetLink", LibLordFarlander_ItemList.GetLinkFromListEntry, "GetTexture", LibLordFarlander_ItemList.GetTextureFromListEntry, "GetID", LibLordFarlander_ItemList.GetIDFromListEntry )
end--LibLordFarlander_ItemList.BuildBasicItem()

function LibLordFarlander_ItemList.BuildBagItem( bag, slot )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    returnVal.bag = bag
    returnVal.slot = slot
    returnVal.GetLink = GetLink_BagItem
    returnVal.GetName = GetName_BagItem
    returnVal.GetTexture = GetTexture_BagItem
    return returnVal
end--LibLordFarlander_ItemList.BuildBagItem( bag, slot )

function LibLordFarlander_ItemList.BuildInventoryItem( invslot )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( type( invslot ) == string ) then
        invslot = GetInventorySlotInfo( invslot )
    end--if
    returnVal.invslot = invslot
    returnVal.GetLink = GetLink_InventoryItem
    returnVal.GetName = GetName_InventoryItem
    returnVal.GetTexture = GetTexture_InventoryItem
    return returnVal
end--LibLordFarlander_ItemList.BuildInventoryItem( invslot )

function LibLordFarlander_ItemList.BuildSpell( spellID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( spellID < 0 ) then
        spellID = spellID * -1
    end--if
    returnVal.spellID = spellID
    returnVal.GetLink = GetLink_Spell
    returnVal.GetName = GetName_Spell
    returnVal.GetTexture = GetTexture_Spell
    return returnVal
end--LibLordFarlander_ItemList.BuildSpell( spellID )

function LibLordFarlander_ItemList.BuildItem( itemID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( type( itemID ) == "string" ) then
        itemID = LibLordFarlander.GetItemIDFromLink( LibLordFarlander.GetItemLink( itemID ) )
    end--if
    if( itemID < 0 ) then
        return LibLordFarlander_ItemList.BuildSpell( itemID * -1 )
    end--if
    returnVal.itemID = itemID
    returnVal.GetLink = GetLink_Item
    returnVal.GetName = GetName_Item
    returnVal.GetTexture = GetTexture_Item
    return returnVal
end--LibLordFarlander_ItemList.BuildItem( itemID )

function LibLordFarlander_ItemList.BuildBattlePet( petID, fullPetID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( petID < 0 ) then
        petID = petID * -1
    end--if
    returnVal.petID = petID
    returnVal.fullPetID = fullPetID
    returnVal.GetLink = GetLink_BattlePet
    returnVal.GetName = GetName_BattlePet
    returnVal.GetTexture = GetTexture_BattlePet
    return returnVal
end--LibLordFarlander_ItemList.BuildBattlePet( petID, fullPetID )

function LibLordFarlander_ItemList.BuildMount( mountID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( mountID < 0 ) then
        mountID = mountID * -1
    end--if
    returnVal.mountID = mountID
    returnVal.GetLink = GetLink_Mount
    returnVal.GetName = GetName_Mount
    returnVal.GetTexture = GetTexture_Mount
    return returnVal
end--LibLordFarlander_ItemList.BuildMount( mountID )

function LibLordFarlander_ItemList.BuildToy( toyID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    if( toyID < 0 ) then
        toyID = toyID * -1
    end--if
    returnVal.toyID = toyID
    returnVal.GetLink = GetLink_Toy
    returnVal.GetName = GetName_Toy
    returnVal.GetTexture = GetTexture_Toy
    return returnVal
end--LibLordFarlander_ItemList.BuildToy( toyID )

function LibLordFarlander_ItemList.BuildFollower( followerID )
    local returnVal = LibLordFarlander_ItemList.BuildBasicItem()

    returnVal.followerID = followerID
    returnVal.GetLink = GetLink_Follower
    returnVal.GetName = GetName_Follower
    returnVal.GetTexture = GetTexture_Follower
    return returnVal
end--LibLordFarlander_ItemList.BuildFollower( followerID )

function LibLordFarlander_ItemList.AddBagItemToList( list, bag, slot )
    local item = LibLordFarlander_ItemList.BuildBagItem( bag, slot )

    table.insert( list, item )
    return item;
end--LibLordFarlanderLibLordFarlander_ItemList.AddBagItemToList( list, bag, slot )

function LibLordFarlander_ItemList.AddInventoryItemToList( list, invslot )
    local item = LibLordFarlander_ItemList.BuildInventoryItem( invslot )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddBagItemToList( list, bag, slot )

function LibLordFarlander_ItemList.AddSpellToList( list, spellID )
    local item = LibLordFarlander_ItemList.BuildSpell( spellID )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddSpellToList( list, spellID )

function LibLordFarlander_ItemList.AddItemToList( list, itemID )
    if( itemID < 0 ) then
        return LibLordFarlander_ItemList.AddSpellToList( list, itemID * -1 )
    else
        local item = LibLordFarlander_ItemList.BuildItem( itemID )

        table.insert( list, item )
        return item
    end--if
end--LibLordFarlander_ItemList.AddItemToList( list, spellID )

function LibLordFarlander_ItemList.AddBattlePetToList( list, petID )
    local item = LibLordFarlander_ItemList.BuildBattlePet( petID )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddBattlePetToList( list, petID )

function LibLordFarlander_ItemList.AddMountToList( list, mountID )
    local item = LibLordFarlander_ItemList.BuildMount( mountID )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddMountToList( list, mountID )

function LibLordFarlander_ItemList.AddToyToList( list, toyID )
    local item = LibLordFarlander_ItemList.BuildToy( toyID )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddToyToList( list, toyID )

function LibLordFarlander_ItemList.AddFollowerToList( list, followerID )
    local item = LibLordFarlander_ItemList.AddFollowerToList( followerID )

    table.insert( list, item )
    return item
end--LibLordFarlander_ItemList.AddFollowerToList( list, followerID )

function LibLordFarlander_ItemList.GetListEntryType( item )
    if( item.spellID ) then
        --spell
        return "spell"
    elseif( item.invslot ) then
        --inventory
        return "inventoryslot"
    elseif( item.itemID ) then
        --item
        return "item"
    elseif( item.bag and item.slot ) then
        --bag slot
        return "baggeditem"
    elseif( item.petID ) then
        --battle pet
        return "battlepet"
    elseif( item.mountID ) then
        --mount
        return "mount"
    elseif( item.toyID ) then
        --toy
        return "toy"
    elseif( item.followerID ) then
        --follower
        return "follower"
    end--if
    return nil
end--LibLordFarlander_ItemList.GetListEntryType( item )

function LibLordFarlander_ItemList.FindInBags( itemID )
    local GetItemIDFromLink = LibLordFarlander.GetItemIDFromLink

    for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            local itemLink = GetContainerItemLink( bag, slot )

            if( itemLink ) then
                if( itemID == GetItemIDFromLink( itemLink ) ) then
                    return LibLordFarlander_ItemList.BuildBagItem( bag, slot )
                end--if
            end--if
        end--for
    end--for
    return nil
end--LibLordFarlander_ItemList.FindInBags( itemID )

function LibLordFarlander_ItemList.GetNameFromListEntry( item )
    if( item.spellID ) then
        --spell
        return GetName_Spell( item )
    elseif( item.invslot ) then
        --inventory
        return GetName_InventoryItem( item )
    elseif( item.itemID ) then
        --item
        return GetName_Item( item )
    elseif( item.bag and item.slot ) then
        --bag slot
        return GetName_BagItem( item )
    elseif( item.petID ) then
        --battle pet
        return GetName_BattlePet( item )
    elseif( item.mountID ) then
        --mount
        return GetName_Mount( item )
    elseif( item.toyID ) then
        --toy
        return GetName_Toy( item )
    elseif( item.followerID ) then
        --follower
        return GetName_Follower( item )
    end--if
    return nil
end--LibLordFarlander_ItemList.GetNameFromListEntry( item )

function LibLordFarlander_ItemList.GetLinkFromListEntry( item )
    if( item.spellID ) then
        --spell
        return GetLink_Spell( item )
    elseif( item.invslot ) then
        --inventory
        return GetLink_InventoryItem( item )
    elseif( item.itemID ) then
        --item
        return GetLink_Item( item )
    elseif( item.bag and item.slot ) then
        --bag slot
        return GetLink_BagItem( item )
    elseif( item.petID ) then
        --battle pet
        return GetLink_BattlePet( item )
    elseif( item.mountID ) then
        --mount
        return GetLink_Mount( item )
    elseif( item.toyID ) then
        --toy
        return GetLink_Toy( item )
    elseif( item.followerID ) then
        --follower
        return GetLink_Follower( item )
    end--if
    return nil
end--LibLordFarlander_ItemList.GetNameFromListEntry( item )

function LibLordFarlander_ItemList.GetIDFromListEntry( item )
    if( item.spellID ) then
        --spell
        return item.spellID
    elseif( item.invslot ) then
        --inventory
        return LibLordFarlander.GetItemIDFromLink( GetInventoryItemLink( "player", item.invslot ) )
    elseif( item.itemID ) then
        --item
        return item.itemID
    elseif( item.bag and item.slot ) then
        --bag slot
        return LibLordFarlander.GetItemIDFromLink( GetContainerItemLink( item.bag, item.slot ) )
    elseif( item.petID ) then
        --battle pet
        return item.petID
    elseif( item.mountID ) then
        --mount
        local _, spellID = C_MountJournal.GetMountInfo( item.mountID )

        return spellID
    elseif( item.toyID ) then
        --toy
        return item.toyID
    elseif( item.followerID ) then
        --follower
        return item.followerID
    end--if
    return nil
end--LibLordFarlander_ItemList.GetIDFromListEntry( item )

function LibLordFarlander_ItemList.GetNormalizedIDFromListEntry( item )
    if( item.spellID ) then
        --spell
        return item.spellID * -1
    elseif( item.invslot ) then
        --inventory
        return LibLordFarlander.GetItemIDFromLink( GetInventoryItemLink( "player", item.invslot ) )
    elseif( item.itemID ) then
        --item
        return item.itemID
    elseif( item.bag and item.slot ) then
        --bag slot
        return LibLordFarlander.GetItemIDFromLink( GetContainerItemLink( item.bag, item.slot ) )
    elseif( item.petID ) then
        --battle pet
        return item.petID
    elseif( item.mountID ) then
        --mount
        local _, spellID = C_MountJournal.GetMountInfo( item.mountID )

        return spellID * -1
    elseif( item.toyID ) then
        --toy
        return item.toyID
    elseif( item.followerID ) then
        --follower
        return item.followerID
    end--if
    return nil
end--LibLordFarlander_ItemList.GetNormalizedIDFromListEntry( item )

function LibLordFarlander_ItemList.GetTextureFromListEntry( item )
    local sTexture = nil

    if( item.spellID ) then
        --spell
        return GetTexture_Spell( item )
    elseif( item.invslot ) then
        --inventory
        return GetTexture_InventoryItem( item )
    elseif( item.itemID ) then
        --item
        return GetTexture_Spell( item )
    elseif( item.bag and item.slot ) then
        --bag slot
        return GetTexture_BagItem( item )
    elseif( item.fullPetID ) then
        --battle pet
        return GetTexture_BattlePet( item )
    elseif( item.mountID ) then
        --mount
        return GetTexture_Mount( item )
    elseif( item.toyID ) then
        --toy
        return GetTexture_Toy( item )
    elseif( item.followerID ) then
        --follower
        return GetTexture_Follower( item )
    end--if
    return sTexture
end--LibLordFarlander_ItemList.GetTextureFromListEntry( item )

function LibLordFarlander_ItemList.GetRandomItemFromList( list, currentItem )
    if( type( list ) == "table" ) then
        local amountOfItems = #list

        if( amountOfItems > 0 ) then
            --math.random( upper ) generates integer numbers between 1 and upper.
            local randomItemIndex = math.random( amountOfItems )

            if( amountOfItems > 1 ) then --Lets make sure the player gets a different item if he has more then one, the quick and dirty way.
                while( LibLordFarlander.CompareTables( list[randomItemIndex], currentItem ) ) do
                    randomItemIndex = math.random( amountOfItems )
                end--while
            end--if
            return list[randomItemIndex]
        end--if
    end--if
    return nil
end--LibLordFarlander_ItemList.GetRandomItemFromList( list, currentItem )

function LibLordFarlander_ItemList.SetTooltipFromListEntry( tooltip, item )
    if( item.spellID ) then
        --spell
        tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
    elseif( item.invslot ) then
        --inventory
        tooltip:SetInventoryItem( "player", item.invslot )
    elseif( item.itemID ) then
        --item
         tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
    elseif( item.bag and item.slot ) then
        --bag slot
        tooltip:SetBagItem( item.bag, item.slot )
    elseif( item.petID ) then
        --battle pet
        -- tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
        tooltip:SetText( item:GetName() )
     elseif( item.mountID ) then
        --mount
        tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
    elseif( item.toyID ) then
        --toy
        tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
    elseif( item.followerID ) then
        --toy
        tooltip:SetHyperlink( LibLordFarlander_ItemList.GetLinkFromListEntry( item ) )
    end--if
end--LibLordFarlander_ItemList.SetTooltipFromListEntry( item )

function LibLordFarlander_ItemList.SetButtonFromListEntry( button, item, setCooldown, cooldownIndicator, macro )
    local sName, itemLink, sItemTexture
    local startTime, duration, enable

    if( not cooldownIndicator ) then
        cooldownIndicator = button.cooldown
    end--if
    if( macro ) then
        button:SetAttribute( "*type1", "macro" )
        button:SetAttribute( "type1", "macro" )
        button:SetAttribute( "shift-type1", "macro" )
        button:SetAttribute( "alt-type1", "macro" )
        button:SetAttribute( "ctrl-type1", "macro" )
        button:SetAttribute( "macrotext", macro )
    else
        button:SetAttribute( "type1", nil )
        button:SetAttribute( "shift-type1", nil )
        button:SetAttribute( "alt-type1", nil )
        button:SetAttribute( "ctrl-type1", nil )
    end--if
    if( item.petID ) then
        --battle pet
        local _, customName, _, _, _, _, _, petName, petIcon = C_PetJournal.GetPetInfoByPetID( ( item.fullPetID or ("0x%016x"):format( item.petID ) ) )

        sItemTexture = petIcon
        sName = customName or petName
        startTime = 0
        duration = 0
        enable = 1
        -- Only summonable by macro
    elseif( item.spellID ) then
        --spell
        local _

        sName, _, sItemTexture = GetSpellInfo( item.spellID )
        startTime, duration, enable = GetSpellCooldown( sName, BOOKTYPE_SPELL )
        if( not macro ) then
            button:SetAttribute( "*type1", "spell" )
            button:SetAttribute( "*spell1", sName )
            button:SetAttribute( "*bag1", nil )
            button:SetAttribute( "*slot1", nil )
            button:SetAttribute( "*item1", nil )
        end--if
    elseif( item.invslot ) then
        --inventory
        itemLink = GetInventoryItemLink( "player", item.invslot )
        if( itemLink ) then
            local _

            sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( itemLink )
            startTime, duration, enable = GetItemCooldown( LibLordFarlander.GetItemIDFromLink( itemLink ) )
        end--if
        if( not macro ) then
            button:SetAttribute( "*type1", "item" )
            button:SetAttribute( "*bag1", nil )
            button:SetAttribute( "*slot1", item.invslot )
            button:SetAttribute( "*item1", nil )
        end--if
    elseif( item.itemID ) then
        --item
        local _

        sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( item.itemID )
        startTime, duration, enable = GetItemCooldown( item.itemID )
        if( not macro ) then
            button:SetAttribute( "*type1", "item" )
            button:SetAttribute( "*bag1", nil )
            button:SetAttribute( "*slot1", nil )
            button:SetAttribute( "*item1", sName )
        end--if
    elseif( item.bag and item.slot ) then
        --bag slot
        itemLink = GetContainerItemLink( item.bag, item.slot )
        if( itemLink ) then
            local _

            sName, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( itemLink )
            startTime, duration, enable = GetItemCooldown( LibLordFarlander.GetItemIDFromLink( itemLink ) )
        end--if
        if( not macro ) then
            button:SetAttribute( "*type1", "item" )
            button:SetAttribute( "*bag1", item.bag )
            button:SetAttribute( "*slot1", item.slot )
            button:SetAttribute( "*item1", nil )
        end--if
    elseif( item.mountID ) then
        --mount
        local _

        sName, _, sItemTexture = C_MountJournal.GetMountInfo( item.mountID )
        startTime = 0
        duration = 0
        enable = 1
        -- Only summonable by macro
    elseif( item.toyID ) then
        --toy
        local _

        _, sName, sItemTexture = C_ToyBox.GetToyInfo( item.toyID )
        if( not macro ) then
            button:SetAttribute( "*type1", "spell" )
            button:SetAttribute( "*spell1", sName )
            button:SetAttribute( "*bag1", nil )
            button:SetAttribute( "*slot1", nil )
            button:SetAttribute( "*item1", nil )
        end--if
    -- Follower probably doesn't make sense.
    end--if
    if( button.SetImage ) then
        button:SetImage( sItemTexture )
    elseif( LibLordFarlander_UI ) then
        LibLordFarlander_UI.SetButtonImage( button, sItemTexture )
    end--if
    if( enable and setCooldown and ( startTime > 0 ) and ( duration > 0 ) ) then
        cooldownIndicator:Show()
        cooldownIndicator:SetCooldown( startTime, duration )
    else
        cooldownIndicator:Hide()
    end--if
    if( button.class and button.class.ldbobj ) then
        button.class.ldbobj.icon = sItemTexture or ""
        button.class.ldbobj.text = sName or ""
    end--if
end--LibLordFarlander_ItemList.SetButtonFromListEntry( button, item, setCooldown, cooldownIndicator, macro )

function LibLordFarlander_ItemList.AttachToButton( button )
    button.tooltip.SetFromListEntry = LibLordFarlander_ItemList.SetTooltipFromListEntry
    button.SetFromListEntry = LibLordFarlander_ItemList.SetButtonFromListEntry
end--LibLordFarlander_ItemList.AttachToButton( button )
